@@ -24,6 +24,8 @@
 #include <libgtkhtml/layout/htmlboxinline.h>
 #include <libgtkhtml/layout/htmlboxtext.h>
 
+#undef DUMP_BOXES
+
 static void       html_view_accessible_class_init          (HtmlViewAccessibleClass *klass);
 static void       html_view_accessible_finalize            (GObject            *obj);
 static void       html_view_accessible_initialize          (AtkObject          *obj,
@@ -39,18 +41,15 @@ static AtkObject* html_view_accessible_get_focus_object    (GtkWidget          *
 
 static void       focus_object_destroyed                   (gpointer           data);       
 static void       root_object_destroyed                    (gpointer           data);
-static void       set_focus_object                         (GObject            *obj,
-                                                            AtkObject          *focus_obj);       
 static void       set_root_object                          (GObject            *obj,
                                                             HtmlBox            *root);
-static void       html_view_accessible_size_allocate       (GtkWidget          *widget,
-                                                            GtkAllocation      *allocation);
 
 static gpointer parent_class = NULL;
 
 static const gchar* gail_focus_object = "gail-focus-object";
 static const gchar* html_root = "html_root";
 
+#ifdef DUMP_BOXES
 static void
 debug_dump_boxes (HtmlBox *root, gint indent, gboolean has_node, xmlNode *n)
 {
@@ -78,17 +77,24 @@ debug_dump_boxes (HtmlBox *root, gint indent, gboolean has_node, xmlNode *n)
 		g_print ("%s ", root->dom_node->xmlnode->name);
 	g_print ("\n");
 	if (HTML_IS_BOX_TEXT (root)) {
+		HtmlBoxText *box_text;
 		gint len;
-		gchar *text = html_box_text_get_text (HTML_BOX_TEXT (root), &len);
+		gchar *text;
+
+		box_text = HTML_BOX_TEXT (root);
+		text = html_box_text_get_text (box_text, &len);
+		g_print ("Master: %p forced_newline: %d\n", box_text->master, box_text->forced_newline);
   		if (len) {
                         gchar *buffer;
 
 			for (i = 0; i < indent; i++)
 				g_print (" ");
-			buffer = g_malloc (len + 2);
-			strncpy (buffer, text, len);
-			buffer[len] = '\n';
-			buffer[len + 1] = 0;
+			buffer = g_malloc (len + 4);
+			buffer[0] = '|';
+			strncpy (buffer + 1, text, len);
+			buffer[len + 1] = '|';
+			buffer[len + 2] = '\n';
+			buffer[len + 3] = 0;
 			g_print (buffer);
 			g_free (buffer);
 		}
@@ -99,6 +105,7 @@ debug_dump_boxes (HtmlBox *root, gint indent, gboolean has_node, xmlNode *n)
 	  box = box->next;
 	}
 }
+#endif
 
 GType
 html_view_accessible_get_type (void)
@@ -176,7 +183,6 @@ html_view_accessible_initialize (AtkObject *obj,
                                  gpointer  data)
 {
 	GtkWidget *widget;
-	AtkObject *focus_object;
 	HtmlView *view;
 
 	ATK_OBJECT_CLASS (parent_class)->initialize (obj, data);
@@ -184,19 +190,9 @@ html_view_accessible_initialize (AtkObject *obj,
 	widget = GTK_WIDGET (data);
  	view = HTML_VIEW (data);
 	set_root_object (G_OBJECT (obj), view->root);
-	g_signal_connect (widget, "grab_focus",
-			  G_CALLBACK (html_view_accessible_grab_focus_cb),
-                          NULL);
-	/*
-	 * We use the size_allocate signal handler to check whether the
-	 * root node of the HtmlView has changed.
-	 */
-	g_signal_connect (widget, "size_allocate",
-			  G_CALLBACK (html_view_accessible_size_allocate),
-                          NULL);
-	focus_object = html_view_accessible_get_focus_object (widget, NULL);
-	if (focus_object)
-		set_focus_object (G_OBJECT (obj), focus_object);
+	g_signal_connect_after (widget, "grab_focus",
+			        G_CALLBACK (html_view_accessible_grab_focus_cb),
+                                NULL);
 }
 
 static void
@@ -264,9 +260,17 @@ html_view_accessible_ref_child (AtkObject *obj, gint i)
 	html_box = html_view->root;
 
 	if (html_box) {
+		gpointer old_root;
+
 		atk_child = atk_gobject_accessible_for_object (G_OBJECT (html_box));
 		g_object_set_data (G_OBJECT (html_box), "view", widget);
 		g_object_ref (atk_child);
+		/* We check whether the root node has changed */	
+		old_root = g_object_get_data (G_OBJECT (obj), html_root);
+		if (!old_root) {
+			set_root_object (G_OBJECT (obj), html_box);
+			g_signal_emit_by_name (obj, "children_changed::add", 0, NULL, NULL);
+		}
 	}
 	return atk_child;
 }
@@ -281,10 +285,13 @@ html_view_accessible_grab_focus_cb (GtkWidget *widget)
 	focus_object = html_view_accessible_get_focus_object (widget, &link_index);
 
 	obj = gtk_widget_get_accessible (widget);
-	set_focus_object (G_OBJECT (obj), focus_object);
-	if (GTK_WIDGET_HAS_FOCUS (widget) && focus_object) {
-		atk_focus_tracker_notify (focus_object);
-		g_signal_emit_by_name (focus_object, "link-selected", link_index);
+	if (GTK_WIDGET_HAS_FOCUS (widget)) {
+		if (focus_object) {
+			atk_focus_tracker_notify (focus_object);
+			g_signal_emit_by_name (focus_object, "link-selected", link_index);
+		} else {
+			atk_focus_tracker_notify (obj);
+		}
 	}
 }
 
@@ -295,7 +302,7 @@ get_link_index (HtmlBox *root, HtmlBox *link_box, gint *link_index)
 	gboolean ret;
 
 	if (!root)
-		return NULL;
+		return FALSE;
 
 	if (HTML_IS_BOX_INLINE (root)) {
 		if (root == link_box)
@@ -340,14 +347,20 @@ html_view_accessible_get_focus_object (GtkWidget *widget, gint *link_index)
 			}
 		}
 		g_assert (HTML_IS_BOX_BLOCK (box));
-
+		if (box->dom_node && strcmp ((char *)box->dom_node->xmlnode->name, "p") == 0) {
+			if (link_index) {
+				index = 0;
+				if (get_link_index (box, focus_box, &index))
+					*link_index = index;
+			}
+		} else {
+			box = focus_box->children;
+			if (link_index) {
+				*link_index = 0;
+			}
+		}
 		g_object_set_data (G_OBJECT (box), "view", widget);
 		atk_obj = atk_gobject_accessible_for_object (G_OBJECT (box));
-		if (link_index) {
-			index = 0;
-			if (get_link_index (box, focus_box, &index))
-				*link_index = index;
-		}
 	} else {
 		atk_obj = NULL;
 	}
@@ -368,7 +381,7 @@ html_view_accessible_ref_state_set (AtkObject *obj)
 
 	view = HTML_VIEW (widget);
 
-#if 0
+#ifdef DUMP_BOXES 
 	debug_dump_boxes (view->root, 0, FALSE, NULL);
 #endif
 
@@ -391,25 +404,6 @@ root_object_destroyed (gpointer data)
 	g_signal_emit_by_name (data, "children_changed::remove", 0, NULL, NULL);
 }
 
-static void
-set_focus_object (GObject *obj,
-                  AtkObject *focus_obj) 
-{
-	gpointer old_focus_obj;
-
-	old_focus_obj = g_object_get_data (obj, gail_focus_object);
-	if (old_focus_obj) {
-		g_object_weak_unref (old_focus_obj, 
-				     (GWeakNotify) focus_object_destroyed,
-				     obj);
-	}
-	if (focus_obj)
-		g_object_weak_ref (G_OBJECT (focus_obj), 
-				   (GWeakNotify) focus_object_destroyed,
-				   obj);
-	g_object_set_data (obj, gail_focus_object, focus_obj);
-}
-
 static void
 set_root_object (GObject *obj,
                  HtmlBox *root) 
@@ -430,22 +424,3 @@ set_root_object (GObject *obj,
 	g_object_set_data (obj, html_root, root);
 }
 
-static void
-html_view_accessible_size_allocate (GtkWidget	*widget,
-                                    GtkAllocation *allocation)
-{
-	HtmlView *view;
-	AtkObject *obj;
-	gpointer old_root;
-
-	view = HTML_VIEW (widget);
-	obj = gtk_widget_get_accessible (widget);
-
-	/* We check whether the root node has changed */	
-	old_root = g_object_get_data (G_OBJECT (obj), html_root);
-	if (!old_root && view->root) {
-		set_root_object (G_OBJECT (obj), view->root);
-		g_signal_emit_by_name (obj, "children_changed::add", 0, NULL, NULL);
-	}
-}
-
