@@ -103,7 +103,7 @@ css_matcher_match_simple_selector (CssSimpleSelector *simple, xmlNode *node, Htm
 				const gchar *beg, *end, *next, *matchstr;
 				gint matchlen;
 				
-				matchstr = html_atom_list_get_string (html_atom_list, tail->t.class_sel.class);
+				matchstr = html_atom_list_get_string (html_atom_list, tail->t.class_sel.css_class);
 				matchlen = strlen (matchstr);
 
 				for (beg = element_class; beg != NULL; beg = next) {
@@ -596,7 +596,13 @@ handle_background_image (HtmlDocument *document, HtmlStyle *style, CssValue *val
 			g_object_unref (G_OBJECT(image));
 			return TRUE;
 		}
-	} 
+	} else if (val->value_type == CSS_IDENT &&
+		   val->v.atom == HTML_ATOM_NONE) {
+		if (style->background->image) {
+			g_object_unref (style->background->image);
+			style->background->image = NULL;
+		}
+	}
 	return FALSE;
 }
 
@@ -1256,7 +1262,10 @@ css_matcher_apply_rule (HtmlDocument *document, HtmlStyle *style, HtmlStyle *par
 		}
 		else if (val->value_type == CSS_FUNCTION)
 			handle_background_image (document, style, val);
-		else if (css_parse_color (val, &color))
+		else if (val->value_type == CSS_IDENT && val->v.atom == HTML_ATOM_NONE) {
+			handle_background_image (document, style, val);
+			style->background->color.transparent = TRUE;
+		} else if (css_parse_color (val, &color))
 			html_style_set_background_color (style, &color);
 
 		break;
@@ -1273,6 +1282,8 @@ css_matcher_apply_rule (HtmlDocument *document, HtmlStyle *style, HtmlStyle *par
 		
 		if (val->v.atom == HTML_ATOM_INHERIT)
 			html_style_set_background_color (style, &parent_style->background->color);
+		else if (val->v.atom == HTML_ATOM_NONE)
+			style->background->color.transparent = TRUE;
 		else if (css_parse_color (val, &color))
 			html_style_set_background_color (style, &color);
 		break;
@@ -1310,11 +1321,8 @@ css_matcher_apply_rule (HtmlDocument *document, HtmlStyle *style, HtmlStyle *par
 		case HTML_ATOM_JUSTIFY:
 			type = HTML_TEXT_ALIGN_JUSTIFY;
 			break;
-		case HTML_ATOM_BOTTOM:
-			g_warning ("text-align:bottom not supported");
-			break;
 		default:
-			g_assert_not_reached ();
+			g_message ("Unrecognized text-align attribute value");
 			break;
 		};
 		html_style_set_text_align (style, type);
@@ -1385,9 +1393,10 @@ css_matcher_apply_rule (HtmlDocument *document, HtmlStyle *style, HtmlStyle *par
 			while (entry) {
 				CssValue *value = entry->value;
 				gchar *value_str = css_value_to_string (value);
-				len += strlen (value_str);
-				if (value_str)
+				if (value_str) {
+					len += strlen (value_str);
 					g_free (value_str);
+				}
 				/* Add space for the ',' character */
 				if (entry->next)
 					len++;
@@ -1401,11 +1410,12 @@ css_matcher_apply_rule (HtmlDocument *document, HtmlStyle *style, HtmlStyle *par
 			while (entry) {
 				CssValue *value = entry->value;
 				gchar *value_str = css_value_to_string (value);
-				strcat (str, value_str);
-				if (value_str)
+				if (value_str) {
+					strcat (str, value_str);
 					g_free (value_str);
-				if (entry->next)
-					strcat (str, ",");
+					if (entry->next)
+						strcat (str, ",");
+				}
 				entry = entry->next;
 			}
 			html_style_set_font_family (style, str);
@@ -2354,7 +2364,7 @@ css_matcher_apply_rule (HtmlDocument *document, HtmlStyle *style, HtmlStyle *par
 		break;
 		
 	default:
-		g_warning ("Unhandled property: %d %s\n", prop, html_atom_list_get_string (html_atom_list, prop));
+		g_print ("Unhandled property: %d %s\n", prop, html_atom_list_get_string (html_atom_list, prop));
 	}
 }
 
@@ -2487,7 +2497,7 @@ static void
 css_matcher_html_to_css (HtmlDocument *document, HtmlStyle *style, xmlNode *n)
 {
 	gchar *str;
-	
+
 	if (n->type != XML_ELEMENT_NODE)
 		return;
 
@@ -2586,7 +2596,7 @@ css_matcher_html_to_css (HtmlDocument *document, HtmlStyle *style, xmlNode *n)
 			xmlFree (str);
 		}
 		if ((str = xmlGetProp (n, "size"))) {		
-
+			gfloat min_size;
 			gint font_size;
 
 			/* Relative size */
@@ -2599,7 +2609,25 @@ css_matcher_html_to_css (HtmlDocument *document, HtmlStyle *style, xmlNode *n)
 			else
 				font_size = atoi (str);
 
-			html_style_set_font_size_html (style, font_size);
+			g_object_get (gtk_settings_get_default (),
+				      "gtkhtml-minimum-font-size",
+				      &min_size, NULL);
+			if (html_font_description_html_size_to_pt (font_size)
+			    < min_size) {
+				HtmlFontSpecification *min_size_spec;
+
+				min_size_spec = html_font_specification_new ("min_size",
+									     HTML_FONT_STYLE_NORMAL,
+									     HTML_FONT_VARIANT_NORMAL,
+									     HTML_FONT_WEIGHT_100,
+									     HTML_FONT_STRETCH_NORMAL,
+									     HTML_FONT_DECORATION_NONE,
+									     min_size);
+				html_style_set_font_size_html (style, html_font_specification_get_html_size (min_size_spec));
+				html_font_specification_unref (min_size_spec);
+			} else {
+				html_style_set_font_size_html (style, font_size);
+			}
 
 			xmlFree (str);
 		}
@@ -2736,7 +2764,7 @@ css_matcher_html_to_css_after (HtmlStyle *style, xmlNode *n)
 		}
 	}
 	if (strcasecmp ("table", n->name) == 0) {
-		if ((str = xmlGetProp (n, "align"))) {
+		if ((style->display != HTML_DISPLAY_BLOCK) && (str = xmlGetProp (n, "align"))) {
 			if (strcasecmp (str, "left") == 0)
 				style->Float = HTML_FLOAT_LEFT;
 			else if (strcasecmp (str, "right") == 0)
