@@ -68,7 +68,8 @@ css_parser_parse_whitespace (const gchar *buffer, gint start_pos, gint end_pos)
 	return pos;
 }
 
-
+/* Find a particular character in the CSS buffer, skipping over quoted strings and blocks of
+   code surrounded by parenthesis and braces */
 static gint
 css_parser_parse_to_char (const gchar *buffer, guchar ch, gint start_pos, gint end_pos)
 {
@@ -80,14 +81,12 @@ css_parser_parse_to_char (const gchar *buffer, guchar ch, gint start_pos, gint e
 	while (pos < end_pos) {
 		guchar c = buffer[pos];
 
-		if (c == '\"')
+		if (c == '\"' && !in_single_quote)
 			in_double_quote = !in_double_quote;
-		else if (c == '\'')
+		else if (c == '\'' && !in_double_quote)
 			in_single_quote = !in_single_quote;
-		else if (c == ch) {
-			if (!in_double_quote && !in_single_quote) 
-				break;
-		}
+		else if (c == ch && !in_double_quote && !in_single_quote)
+			break;
 		else if (c == '{' && !in_double_quote && !in_single_quote) {
 			tmp_pos = css_parser_parse_to_char (buffer, '}', pos + 1, end_pos);
 			if (tmp_pos == end_pos)
@@ -801,7 +800,7 @@ css_parser_parse_simple_selector (const gchar *buffer, gint start_pos, gint end_
 				tail = g_realloc (tail, sizeof (CssTail) *
 						  (n_tail_max <<= 1));
 			tail [n_tail].type = CSS_TAIL_CLASS_SEL;
-			tail [n_tail].t.class_sel.class = id;
+			tail [n_tail].t.class_sel.css_class = id;
 			n_tail++;
 
 /*			g_print ("class sel!\n"); */
@@ -1184,13 +1183,6 @@ css_parser_prepare_stylesheet (const gchar *str, gint len)
 
 			pos += 2;
 		}
-		else if (str[pos] == '/' &&
-			 pos + 1 <= len &&
-			 str[pos + 1] == '/') {
-			while (pos < len &&
-			       str[pos] != '\n')
-				pos++;
-		}
 		if (buffer_size == buffer_size_max - 1)
 			result = g_realloc (result, (buffer_size_max <<= 1));
 		
@@ -1412,7 +1404,9 @@ css_parser_parse_stylesheet (const gchar *str, gint len)
 	pos = css_parser_parse_whitespace (buffer, pos, len);
 	end_pos = len;
 	while (pos < len) {
-		if (buffer[pos] == '@') {
+		if (buffer[pos] == '@' && pos + 1 < len &&
+		    ((buffer[pos + 1] >= 'a' && buffer[pos + 1] <= 'z') ||
+		     (buffer[pos + 1] >= 'A' && buffer[pos + 1] <= 'Z'))) {
 			pos = css_parser_parse_atkeyword (buffer, pos + 1, len, &statement);
 #if 0
 			if (statement) {
