@@ -21,9 +21,8 @@
 */
 
 #include <string.h>
-
+#include <gtk/gtkicontheme.h>
 #include <graphics/htmlpainter.h>
-#include "images/loading_image.xpm"
 
 static GObjectClass *parent_class = NULL;
 
@@ -41,9 +40,16 @@ html_painter_class_init (HtmlPainterClass *klass)
 static void
 html_painter_init (HtmlPainter *painter)
 {
+	GtkIconTheme *icon_theme;
+
+	icon_theme = gtk_icon_theme_get_default ();
+
 	painter->debug = FALSE;
 
-	painter->loading_picture = gdk_pixbuf_new_from_xpm_data (loading_image_xpm);
+	if (gtk_icon_theme_has_icon (icon_theme, "image-loading"))
+		painter->loading_picture = gtk_icon_theme_load_icon (icon_theme, "image-loading", 16, 0, NULL);
+	else
+		painter->loading_picture = gtk_icon_theme_load_icon (icon_theme, "gnome-fs-loading-icon", 16, 0, NULL);
 }
 
 GType
