@@ -54,7 +54,7 @@ static void
 html_box_embedded_button_set_label (HtmlBoxEmbeddedButton *button)
 {
 	HtmlBoxEmbedded *embedded = HTML_BOX_EMBEDDED (button);
-	const gchar *value = dom_HTMLInputElement__get_value (DOM_HTML_INPUT_ELEMENT (HTML_BOX (embedded)->dom_node));
+	gchar *value = dom_HTMLInputElement__get_value (DOM_HTML_INPUT_ELEMENT (HTML_BOX (embedded)->dom_node));
 
 	if (value && strlen (value))
 		gtk_label_set_text (GTK_LABEL (GTK_BIN (embedded->widget)->child), value);
@@ -68,6 +68,7 @@ html_box_embedded_button_set_label (HtmlBoxEmbeddedButton *button)
 			break;
 		}
 	}
+	g_free (value);
 }
 
 static void
