@@ -26,6 +26,13 @@
 
 static HtmlBoxClass *parent_class = NULL;
 
+static void html_box_embedded_image_resize_image (HtmlImage *image,
+						  HtmlBoxEmbeddedImage *box);
+static void html_box_embedded_image_repaint_image (HtmlImage *image,
+						   gint x, gint y,
+						   gint width, gint height,
+						   HtmlBoxEmbeddedImage *box);
+
 static void
 html_box_embedded_image_paint (HtmlBox *box, HtmlPainter *painter, GdkRectangle *area, gint tx, gint ty)
 {
@@ -63,13 +70,29 @@ html_box_embedded_image_relayout (HtmlBox *box, HtmlRelayout *relayout)
 	box->height = height + html_box_vertical_mbp_sum (box);
 }
 
+static void
+html_box_embedded_image_finalize (GObject *object)
+{
+	HtmlBoxEmbeddedImage *image = HTML_BOX_EMBEDDED_IMAGE (object);
+
+	g_signal_handlers_disconnect_by_func (G_OBJECT (image->image),
+					      G_CALLBACK (html_box_embedded_image_resize_image),
+					      image);
+	g_signal_handlers_disconnect_by_func (G_OBJECT (image->image),
+					      G_CALLBACK (html_box_embedded_image_repaint_image),
+					      image);
+}
+
 static void
 html_box_embedded_image_class_init (HtmlBoxClass *klass)
 {
-	
+	GObjectClass *object_class = (GObjectClass *)klass;
+
 	klass->paint    = html_box_embedded_image_paint;
 	klass->relayout = html_box_embedded_image_relayout;
 
+	object_class->finalize = html_box_embedded_image_finalize;
+
 	parent_class = g_type_class_peek_parent (klass);
 }
 
