@@ -146,6 +146,7 @@ html_box_text_finalize (GObject *self)
 
 	master = text->master;
 	if (master) {
+		html_box_text_destroy_slaves (text);
 		html_box_text_free_master (master);
 		g_free (master);
 		text->master = NULL;
@@ -736,7 +737,7 @@ html_box_text_get_len (HtmlBoxText *box)
 gchar *
 html_box_text_get_text (HtmlBoxText *box, int *text_len)
 {
-	g_return_if_fail (box != NULL);
+	g_return_val_if_fail (box != NULL, NULL);
 
 	if (text_len)
 		*text_len = box->length;
