@@ -85,7 +85,7 @@ html_style_box_dup (HtmlStyleBox *box)
 void
 html_style_set_width (HtmlStyle *style, const HtmlLength *width)
 {
-	if (!(style->box && html_length_equals (&style->box->width, width))) {
+	if (!html_length_equals (&style->box->width, width)) {
 		if (style->box->refcount > 1)
 			html_style_set_style_box (style, html_style_box_dup (style->box));
 		html_length_set (&style->box->width, width);
@@ -95,7 +95,7 @@ html_style_set_width (HtmlStyle *style, const HtmlLength *width)
 void
 html_style_set_height (HtmlStyle *style, const HtmlLength *height)
 {
-	if (!(style->box && html_length_equals (&style->box->height, height))) {
+	if (!html_length_equals (&style->box->height, height)) {
 		if (style->box->refcount > 1)
 			html_style_set_style_box (style, html_style_box_dup (style->box));
 		html_length_set (&style->box->height, height);
@@ -105,7 +105,7 @@ html_style_set_height (HtmlStyle *style, const HtmlLength *height)
 void
 html_style_set_min_width (HtmlStyle *style, const HtmlLength *min_width)
 {
-	if (!(style->box && html_length_equals (&style->box->min_width, min_width))) {
+	if (!html_length_equals (&style->box->min_width, min_width)) {
 		if (style->box->refcount > 1)
 			html_style_set_style_box (style, html_style_box_dup (style->box));
 		html_length_set (&style->box->min_width, min_width);
@@ -115,7 +115,7 @@ html_style_set_min_width (HtmlStyle *style, const HtmlLength *min_width)
 void
 html_style_set_max_width (HtmlStyle *style, const HtmlLength *max_width)
 {
-	if (!(style->box && html_length_equals (&style->box->max_width, max_width))) {
+	if (!html_length_equals (&style->box->max_width, max_width)) {
 		if (style->box->refcount > 1)
 			html_style_set_style_box (style, html_style_box_dup (style->box));
 		html_length_set (&style->box->max_width, max_width);
@@ -126,7 +126,7 @@ html_style_set_max_width (HtmlStyle *style, const HtmlLength *max_width)
 void
 html_style_set_min_height (HtmlStyle *style, const HtmlLength *min_height)
 {
-	if (!(style->box && html_length_equals (&style->box->min_height, min_height))) {
+	if (!html_length_equals (&style->box->min_height, min_height)) {
 		if (style->box->refcount > 1)
 			html_style_set_style_box (style, html_style_box_dup (style->box));
 		html_length_set (&style->box->min_height, min_height);
@@ -137,7 +137,7 @@ html_style_set_min_height (HtmlStyle *style, const HtmlLength *min_height)
 void
 html_style_set_max_height (HtmlStyle *style, const HtmlLength *max_height)
 {
-	if (!(style->box && html_length_equals (&style->box->max_height, max_height))) {
+	if (!html_length_equals (&style->box->max_height, max_height)) {
 		if (style->box->refcount > 1)
 			html_style_set_style_box (style, html_style_box_dup (style->box));
 		html_length_set (&style->box->max_height, max_height);
