@@ -1139,7 +1139,6 @@ html_view_move_backward_word (HtmlView *view, gint start)
 	gint n_attrs;
 	HtmlBoxText *text;
 	gint box_offset;
-	const char *char_text;
 
 	html_view_setup_layout (view);
 	layout = html_view_get_layout (view);
@@ -1200,7 +1199,6 @@ static void
 html_view_get_virtual_cursor_pos (HtmlView *view, gint *x, gint *y)
 {
 	gint virtual_x, virtual_y;
-	gint cursor_x, cursor_y;
 	GdkRectangle pos;
 
 	virtual_x = html_view_get_virtual_cursor_x (view);
@@ -1446,7 +1444,6 @@ html_view_move_cursor_by_line (HtmlView *view, gint count, gint x_pos, gint *off
 {
 	HtmlBoxText *text;
 	HtmlBox *box;
-	HtmlBox *last_box;
 	gint tmp_offset;
 	gint y, height;
 	gint old_y, old_height;
@@ -1569,7 +1566,7 @@ html_view_scroll_pages (HtmlView *view, gint count, gboolean extend_selection)
 	HtmlBox *box;
 	HtmlBoxText *text;
 	GtkAdjustment *adj;
-	gint x, y, height;
+	gint y;
 	gdouble new_value;
 	gdouble old_value;
 	gint cursor_x_pos;
@@ -1908,7 +1905,7 @@ html_view_insert_node (HtmlView *view, DomNode *node)
 
 	g_assert (node->style != NULL);
 
-	new_box = html_box_factory_new_box (view, node);
+	new_box = html_box_factory_new_box (view, node, FALSE);
 	
 	if (new_box) {
 
@@ -2131,6 +2128,47 @@ html_view_style_updated (HtmlDocument *document, DomNode *node, HtmlStyleChange
 	}	
 
 	switch (style_change) {
+	case HTML_STYLE_CHANGE_RECREATE: {
+		HtmlBox *new_box = NULL;
+
+		/* Don't replace boxes where display: none has been set */
+		if (!style || style->display == HTML_DISPLAY_NONE)
+			break;
+
+		new_box = html_box_factory_new_box (view, node, TRUE);
+		g_assert (new_box);
+	
+		new_box->dom_node = node;
+		g_object_add_weak_pointer (G_OBJECT (node), (gpointer *)&(new_box->dom_node));
+		html_box_handle_html_properties (new_box, node->xmlnode);				
+		new_box->next = box->next;
+		new_box->prev = box->prev;
+		new_box->parent = box->parent;
+		
+		if (box->parent && box->parent->children == box)
+				box->parent->children = new_box;
+		if (box->next) box->next->prev = new_box;
+		if (box->prev) box->prev->next = new_box;
+
+		if (box->children) {
+			HtmlBox *child = box->children;
+			while (child) {
+				HtmlBox *temp = child->next;
+				html_box_append_child (new_box, child);
+				child = temp;
+			}
+		}
+		
+		if (view->root == box)
+			view->root = new_box;
+		
+		html_view_remove_layout_box (view, node);
+		g_object_unref (box);
+		
+		html_view_add_layout_box (view, node, new_box);
+		html_view_relayout_callback (document, node, view);
+		break;
+	}
 	case HTML_STYLE_CHANGE_REPAINT:
 		html_view_repaint_callback (document, node, view);
 		break;
@@ -2244,9 +2282,7 @@ html_view_realize (GtkWidget *widget)
 	 * GtkLayout uses the bg color for background but we want
 	 * to use base color.
 	 */	
-	widget->style = gtk_style_copy (widget->style);
-	widget->style->bg[GTK_STATE_NORMAL] = 
-		widget->style->base[GTK_STATE_NORMAL];
+	gtk_widget_modify_bg (widget, GTK_STATE_NORMAL, &widget->style->base[GTK_STATE_NORMAL]);
 	/*
 	 * Store the font size so we can adjust size of HtmlFontSpecification
 	 * if the size changes.
@@ -2491,7 +2527,6 @@ static gboolean
 html_view_focus (GtkWidget *widget, GtkDirectionType direction)
 {
 	HtmlView *view = HTML_VIEW (widget);
-	HtmlBox *box;
 	DomElement *new_focus_element = NULL;
 
         if (view->document == NULL || view->document->dom_document == NULL) {
@@ -3147,9 +3182,10 @@ html_view_style_set (GtkWidget *widget, GtkStyle *previous_style)
 				      "focus-line-width", &focus_width,
 				      NULL);
 
-
- 		widget->style->bg[GTK_STATE_NORMAL] =
- 			widget->style->base[GTK_STATE_NORMAL];
+ 		if (!gdk_color_equal (&widget->style->bg[GTK_STATE_NORMAL], &widget->style->base[GTK_STATE_NORMAL])) {
+			gtk_widget_modify_bg (widget, GTK_STATE_NORMAL, &widget->style->base[GTK_STATE_NORMAL]);
+			return;
+ 		}
 		fsize = pango_font_description_get_size (widget->style->font_desc) / (gfloat) PANGO_SCALE;
 		new_isize = (gint) fsize;
 		old_isize = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (widget), "html-view-font-size"));
